@@ -103,11 +103,13 @@ static int wcmButtonPerNotch(WacomDevicePtr priv, int value, int threshold, int
 	return value % threshold;
 }
 
-static void wcmPanscroll(WacomDevicePtr priv, const WacomDeviceState *ds)
+static void wcmPanscroll(InputInfoPtr pInfo, const WacomDeviceState *ds, int x, int y)
 {
+	WacomDevicePtr priv = (WacomDevicePtr)pInfo->private;
 	WacomCommonPtr common = priv->common;
 	int threshold = common->wcmPanscrollThreshold;
 	int *accumulated_x, *accumulated_y;
+	int delta_x, delta_y;
 
 	if (!(priv->flags & SCROLLMODE_FLAG) || !(ds->buttons & 1))
 		return;
@@ -120,11 +122,19 @@ static void wcmPanscroll(WacomDevicePtr priv, const WacomDeviceState *ds)
 		return;
 	}
 
+	if (!is_absolute(pInfo)) {
+		delta_x = x;
+		delta_y = y;
+	}
+	else {
+		delta_x = (x - priv->oldState.x);
+		delta_y = (y - priv->oldState.y);
+	}
+
 	accumulated_x = &priv->wcmPanscrollState.x;
 	accumulated_y = &priv->wcmPanscrollState.y;
-
-	*accumulated_x += (ds->x - priv->oldState.x);
-	*accumulated_y += (ds->y - priv->oldState.y);
+	*accumulated_x += delta_x;
+	*accumulated_y += delta_y;
 
 	DBG(6, priv, "pan x = %d, pan y = %d\n", *accumulated_x, *accumulated_y);
 
@@ -487,7 +497,7 @@ static void sendCommonEvents(InputInfoPtr pInfo, const WacomDeviceState* ds,
 	int buttons = ds->buttons;
 
 	/* send scrolling events if necessary */
-	wcmPanscroll(priv, ds);
+	wcmPanscroll(pInfo, ds, valuators[0], valuators[1]);
 
 	/* send button events when state changed or first time in prox and button unpresses */
 	if (priv->oldState.buttons != buttons || (!priv->oldState.proximity && !buttons))
@@ -1206,7 +1216,7 @@ static void detectPressureIssue(WacomDevicePtr priv,
 				"On %s(%d) a base pressure of %d persists while the pen is in proximity.\n"
 				"\tThis is > %d percent of the maximum value (%d).\n"
 				"\tThis indicates a worn out pen, it is time to change your tool. Also see:\n"
-				"\thttp://sourceforge.net/apps/mediawiki/linuxwacom/index.php?title=Pen_Wear.\n",
+				"\thttps://github.com/linuxwacom/xf86-input-wacom/wiki/Pen-Wear.\n",
 				priv->pInfo->name, priv->serial, priv->minPressure, LIMIT_LOW_PRESSURE, common->wcmMaxZ);
 	} else if (!priv->oldState.proximity)
 		priv->eventCnt = 0;
@@ -1349,45 +1359,46 @@ static void commonDispatchDevice(InputInfoPtr pInfo,
 				return;
 			}
 
-			/* send other events, such as button/wheel */
 			filtered.x = priv->oldState.x;
 			filtered.y = priv->oldState.y;
 		}
 	}
 
-	/* force out-prox when distance is outside wcmCursorProxoutDist for pucks */
-	if (IsCursor(priv))
+	/* force out-prox when distance from surface exceeds wcmProxoutDist */
+	if (IsTablet(priv) && !is_absolute(pInfo))
 	{
 		/* Assume the the user clicks the puck buttons while
-		 * it is resting on the tablet. This works for both
+		 * it is resting on the tablet (and taps styli onto
+		 * the tablet surface). This works for both
 		 * tablets that have a normal distance scale (protocol
 		 * 5) as well as those with an inverted scale (protocol
 		 * 4 for many many kernel versions).
 		 */
-		if (filtered.buttons)
-			common->wcmMaxCursorDist = filtered.distance;
+		if ((IsCursor(priv) && filtered.buttons) ||
+		    (IsStylus(priv) && filtered.buttons & 0x01))
+			priv->wcmSurfaceDist = filtered.distance;
 
-		DBG(10, common, "Distance over"
+		DBG(10, priv, "Distance over"
 				" the tablet: %d, ProxoutDist: %d current"
-				" min/max %d hard prox: %d\n",
+				" surface %d hard prox: %d\n",
 				filtered.distance,
-				common->wcmCursorProxoutDist,
-				common->wcmMaxCursorDist,
+				priv->wcmProxoutDist,
+				priv->wcmSurfaceDist,
 				ds->proximity);
 
-		if (common->wcmMaxCursorDist) {
+		if (priv->wcmSurfaceDist >= 0) {
 			if (priv->oldState.proximity)
 			{
-				if (abs(filtered.distance - common->wcmMaxCursorDist)
-						> common->wcmCursorProxoutDist)
+				if (abs(filtered.distance - priv->wcmSurfaceDist)
+						> priv->wcmProxoutDist)
 					filtered.proximity = 0;
 			}
 			/* once it is out. Don't let it in until a hard in */
-			/* or it gets inside wcmCursorProxoutDist */
+			/* or it gets inside wcmProxoutDist */
 			else
 			{
-				if (abs(filtered.distance - common->wcmMaxCursorDist) >
-						common->wcmCursorProxoutDist && ds->proximity)
+				if (abs(filtered.distance - priv->wcmSurfaceDist) >
+						priv->wcmProxoutDist && ds->proximity)
 					return;
 				if (!ds->proximity)
 					return;
@@ -1544,7 +1555,7 @@ WacomCommonPtr wcmNewCommon(void)
 	common->wcmMaxTouchY = 1024;       /* max touch Y value */
 	common->wcmMaxStripX = 4096;       /* Max fingerstrip X */
 	common->wcmMaxStripY = 4096;       /* Max fingerstrip Y */
-	common->wcmCursorProxoutDistDefault = PROXOUT_INTUOS_DISTANCE;
+	common->wcmProxoutDistDefault = PROXOUT_INTUOS_DISTANCE;
 			/* default to Intuos */
 	common->wcmSuppress = DEFAULT_SUPPRESS;
 			/* transmit position if increment is superior */
