@@ -206,6 +206,7 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 		case 0x34E: /* MobileStudio Pro 16 */
 		case 0x398: /* MobileStudio Pro 13 */
 		case 0x399: /* MobileStudio Pro 16 */
+		case 0x3AA: /* MobileStudio Pro 16 */
 			TabletSetFeature(priv->common, WCM_LCD);
 			/* fall through */
 
@@ -359,6 +360,7 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 		case 0x37E:/* DTH-2452 Touch */
 		case 0x39A:/* MobileStudio Pro 13 Touch */
 		case 0x39B:/* MobileStudio Pro 16 Touch */
+		case 0x3AC:/* MobileStudio Pro 16 Touch */
 			TabletSetFeature(priv->common, WCM_LCD);
 			break;
 	}
