@@ -1,4 +1,3 @@
-#include <config.h>
 #include <xorg-server.h>
 #include <dix.h>
 #include <os.h>
@@ -8,7 +7,7 @@
 #include <xf86Xinput.h>
 #include <xf86_OSproc.h>
 
-#if HAVE_XF86OPTIONPTR
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 14
 #define OPTTYPE XF86OptionPtr
 #define CONST const
 #else
